@@ -1,7 +1,7 @@
 package Perinci::Sub::To::CLIDocData;
 
-our $DATE = '2015-04-07'; # DATE
-our $VERSION = '0.20'; # VERSION
+our $DATE = '2015-07-04'; # DATE
+our $VERSION = '0.21'; # VERSION
 
 use 5.010001;
 use strict;
@@ -30,25 +30,26 @@ sub _has_cats {
 
 sub _add_category_from_spec {
     my ($cats_spec, $thing, $spec, $noun, $has_cats) = @_;
-    my $cat;
-    my $raw_cat = '';
-    my $order;
+    my @cats;
     for (@{ $spec->{tags} // [] }) {
         my $tag_name = ref($_) ? $_->{name} : $_;
         if ($tag_name =~ /^category:(.+)/) {
-            $raw_cat = $1;
-
-            $cat = ucfirst($1);
+            my $cat = ucfirst($1);
             $cat =~ s/-/ /g;
             $cat .= " " . $noun;
-            $order = 50;
-            last;
+            push @cats, [$cat, 50]; # name, ordering
         }
     }
-    $cat //= $has_cats ? "Other $noun" : ucfirst($noun); # XXX translatable?
-    $order //= 99;
-    $thing->{category} = $cat;
-    $cats_spec->{$cat}{order} //= $order;
+    if (!@cats) {
+        @cats = [$has_cats ? "Other $noun" : ucfirst($noun), 99]; # XXX translatable?
+    }
+
+    # old, will be removed someday
+    $thing->{category} = $cats[0][0];
+    # new/current
+    $thing->{categories} = [map {$_->[0]} @cats];
+
+    $cats_spec->{$_->[0]}{order} //= $_->[1] for @cats;
 }
 
 sub _add_default_from_arg_spec {
@@ -478,7 +479,7 @@ Perinci::Sub::To::CLIDocData - From Rinci function metadata, generate structure
 
 =head1 VERSION
 
-This document describes version 0.20 of Perinci::Sub::To::CLIDocData (from Perl distribution Perinci-Sub-To-CLIDocData), released on 2015-04-07.
+This document describes version 0.21 of Perinci::Sub::To::CLIDocData (from Perl distribution Perinci-Sub-To-CLIDocData), released on 2015-07-04.
 
 =head1 SYNOPSIS
 
@@ -528,6 +529,7 @@ Sample result:
        example_categories => { Examples => { order => 99 } },
        examples => [
          {
+           categories   => ["Examples"],
            category     => "Examples",
            cmdline      => "[[prog]] 'a value' --bool1",
            description  => undef,
@@ -552,6 +554,7 @@ Sample result:
                             summary => "Another bool option",
                             tags => ["category:cat1"],
                           },
+           categories  => ["Cat1 options"],
            category    => "Cat1 options",
            description => undef,
            fqarg       => "bool1",
@@ -567,6 +570,7 @@ Sample result:
                             schema => ["bool", { is => 1 }, {}],
                             tags => ["category:cat1"],
                           },
+           categories  => ["Cat1 options"],
            category    => "Cat1 options",
            description => undef,
            fqarg       => "flag1",
@@ -583,6 +587,7 @@ Sample result:
              schema => ["str", { req => 1 }, {}],
              summary => "A required option as well as positional argument",
            },
+           categories => ["Other options"],
            category => "Other options",
            description => undef,
            fqarg => "str1",
@@ -597,6 +602,7 @@ Sample result:
            alias_spec  => 'fix',
            arg         => "bool1",
            arg_spec    => 'fix',
+           categories  => ["Cat1 options"],
            category    => "Cat1 options",
            description => undef,
            fqarg       => "bool1",
